@@ -107,7 +107,7 @@ class Psycopg:
         self.close_connect(conn, cursor)
         return result
 
-    def select_all(self, sql, args):
+    def select_all(self, sql, args=None):
         """select_all方法用于查找所有记录
 
         Parameters
@@ -143,7 +143,7 @@ class Psycopg:
         conn, cursor = self.get_connect()
         try:
             res = cursor.execute(sql, value)
-            conn.commit(conn, cursor)
+            conn.commit()
             self.close_connect(conn, cursor)
             return res
         except Exception as e:
@@ -221,7 +221,7 @@ class Mysql:
         Returns
         ----------
         """
-        if not args:
+        if args:
             args = tuple(args)
         conn = self.pool.connection(shareable=False)
         cursor = conn.cursor()
