@@ -11,33 +11,31 @@
 -------------------------------------------------
 """
 __author__ = 'Asdil'
-from loguru import logger
 import logging
 
 
-def add(name, level=logging.INFO, log_path=None):
+def simple_init(level='INFO', log_path=None):
     """add方法用于新建一个log
 
     Parameters
     ----------
-    name : str
-        模块名称
-    level: logging.INFO
+    level: str
         日志级别
     log_path: str
         保存路径
     Returns
     ----------
     """
-    logger = logging.getLogger(name)
-    logger.setLevel(level)
+    logger = logging.getLogger()
+    level_dict = {'DEBUG': logging.DEBUG, 'INFO': logging.INFO, 'WARNING': logging.WARNING, 'ERROR': logging.ERROR}
+    logger.setLevel(level_dict[level])
 
     if log_path is not None:
         # create a file handler
         handler = logging.FileHandler(log_path)
         handler.setLevel(level)
         # create a logging format
-        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
+        formatter = logging.Formatter('%(asctime)s | %(levelname)s | %(filename)s:%(funcName)s:%(lineno)s - %(message)s')
         handler.setFormatter(formatter)
 
     # add the handlers to the logger
@@ -45,15 +43,21 @@ def add(name, level=logging.INFO, log_path=None):
     return logger
 
 
-def example():
-    """example方法用于
+def init(log_path, level='INFO'):
+    """init方法用于新建一个log, 但不能在spark上面跑
 
     Parameters
     ----------
-    param : str
-
+    log_path : str
+        路径
+    level : str
+        日志等级
+        
     Returns
     ----------
     """
-    # logger.add("somefile.log", rotation="1 MB", enqueue=True, level="INFO")  # 异步写入
-    return 0
+    from loguru import logger
+    logger.add(log_path, rotation="1 MB", enqueue=True, level=level)
+    return logger
+
+
