@@ -12,28 +12,150 @@
 """
 __author__ = 'Asdil'
 import os
-import pip
 import gzip
 import time
 import shutil
-import zipfile
-import psutil
-import itertools
+import math
 import subprocess
 from tqdm import tqdm
 from datetime import datetime
-from inspect import signature
+from collections import Counter
+
+
+def bins_by_step(start, end, step):
+    """bins_by_step方法用于等距分箱
+
+    Parameters
+    ----------
+    start : float
+        最小值
+    end : float
+        最大值
+    step : float
+        步长
+
+    Returns
+    ----------
+    """
+    if start >= end:
+        raise ValueError(u'start is greater than or equal to end')
+    ret = [start]
+    while True:
+        if ret[-1] + step >= end:
+            if ret[-1] == end:
+                break
+            ret.append(end)
+            break
+        ret.append(ret[-1] + step)
+    return ret
+
+
+def counter(data, n=None):
+    """counter方法用于统计元素数量
+
+    Parameters
+    ----------
+    data : list
+        列表数据
+    n : int
+        前n个元素
+    Returns
+    ----------
+    """
+    ret = Counter(data)
+    if not n:
+        ret = ret.most_common(len(ret))
+    else:
+        ret = ret.most_common(n)
+    return ret
+
+
+def is_nan(data):
+    """is_nan方法用于判断是不是空值
+
+    Parameters
+    ----------
+    data : anytype
+        任何数据类型
+
+    Returns
+    ----------
+    """
+    if data is None:
+        return True
+    try:
+        return math.isnan(data)
+    except:
+        return False
+
+
+def filter_nan(data):
+    """filter_na方法用于过滤None值
+
+    Parameters
+    ----------
+    data : list
+        数据列表
+
+    Returns
+    ----------
+    """
+    return list(filter(lambda x: not is_nan(x), data))
+
+
+def is_type(data, typ):
+    """is_type方法用于判断数据类型
+
+    Parameters
+    ----------
+    data : anydata
+        任何数据
+    typ ： anytype
+        需要确认的数据类型
+    Returns
+    ----------
+    """
+    if type(data) is typ:
+        return True
+    return False
+
+
+def sort_list(data, ind, ascending=True):
+    """sort_list方法用于
+
+    Parameters
+    ----------
+    data : list
+        列表
+    ind : int
+        第几列
+    ascending : pool
+        升序或降序
+    Returns
+    ----------
+    """
+    if ascending:
+        data = sorted(data, key=lambda x: x[ind])
+    else:
+        data = sorted(data, key=lambda x: -x[ind])
+    return data
 
 
 def path_join(path1, path2):
-    """
-    合并两个目录
-    :param path1:  路径
-    :param path2:  文件名
-    :return:
+    """path_join方法用于合并两个或多个目录
+
+    Parameters
+    ----------
+    path1 : str
+        主路径
+    path2 : str or list
+        子路径
+    Returns
+    ----------
     """
     assert isinstance(path1, str)
-    assert isinstance(path2, str)
+    if isinstance(path2, list):
+        path2 = os.sep.join(path2)
     if path1[-1] != os.sep:
         path1 += os.sep
     if path2[0] == os.sep:
@@ -41,27 +163,40 @@ def path_join(path1, path2):
     return path1 + path2
 
 
-def get_files(path, extension=None, key=None):
-    """
-    获取目标目录文件
-    :param path:      路径
-    :param extension: 后缀
-    :param key:       关键字
-    :return:
-    """
-    if extension is not None:
+def get_files(path, extension=None, exclude=None, include=None):
+    """get_files方法用于获取目录文件
+
+        Parameters
+        ----------
+        path : str
+            路径
+        extension : str
+            后缀
+        exclude : str
+            包含不包含某个词
+        include : str
+            包含某个词
+
+        Returns
+        ----------
+        """
+    ret = os.listdir(path)
+    if extension:
         length = -len(extension)
-        ret = [path_join(path, each) for each in os.listdir(
-            path) if each[length:] == extension]
-    elif key is not None:
-        ret = [path_join(path, each)
-               for each in os.listdir(path) if key in each]
-    else:
-        ret = [path_join(path, each) for each in os.listdir(path)]
+        ret = [path_join(path, each) for each in os.listdir(path) if each[length:] == extension]
+        if not ret:
+            return []
+    if exclude:
+        ret = list(filter(lambda x: exclude not in x, ret))
+        if not ret:
+            return []
+    if include:
+        ret = list(filter(lambda x: include in x, ret))
+        if not ret:
+            return []
     return ret
 
 
-# 获取文件名
 def get_name(path, extension=None, key=None):
     """
     获取目标目录下文件名
@@ -252,7 +387,10 @@ def del_dir(path):
     :param path:  路径
     :return:
     """
-    shutil.rmtree(path)
+    if os.path.exists(path):
+        shutil.rmtree(path)
+        return True
+    return False
 
 
 def combin_dic(*args):
@@ -313,6 +451,7 @@ def zip_file(file_path, output=None, rename=None, typ=3):
     :param output:     是否输入到其它文件夹
     :return:           True, False
     """
+    import zipfile
     # 拆分成文件路径，文件
     path, name, _, name_extension = split_path(file_path)
     if rename is None:
@@ -342,6 +481,7 @@ def unzip_file(file_path, output=None):
     :param file_path:  zip文件完整路径
     :return:
     """
+    import zipfile
     path, name, _, name_extension = split_path(file_path)
     azip = zipfile.ZipFile(file_path)
     if output is None:
@@ -574,6 +714,7 @@ def monitor_memery_cpu(pids, second=10, out_path=None, show=False):
     :param show:
     :return:
     """
+    import psutil
     proc = psutil.Process(int(pids))
     info = ['cpu rate\tmemory use']
     while True:
@@ -591,7 +732,7 @@ def monitor_memery_cpu(pids, second=10, out_path=None, show=False):
             f.write('\n'.join(info))
 
 
-def read(path, sep='\n', encoding='utf-8'):
+def read(path, sep='\n', encoding='UTF-8'):
     """
     按行读数据
     :param path: 路径
@@ -599,8 +740,12 @@ def read(path, sep='\n', encoding='utf-8'):
     :param encoding: 编码
     :return:
     """
-    with open(path, 'r', encoding=encoding) as f:
-        return f.read().strip().split(sep)
+    try:
+        with open(path, 'r', encoding=encoding) as f:
+            return f.read().strip().split(sep)
+    except:
+        with open(path, 'r') as f:
+            return f.read().strip().split(sep)
 
 
 def write(data, path, sep='\n', encoding='utf-8'):
@@ -620,8 +765,12 @@ def write(data, path, sep='\n', encoding='utf-8'):
     Returns
     ----------
     """
-    with open(path, 'w', encoding=encoding) as f:
-        f.write(sep.join(data))
+    try:
+        with open(path, 'w', encoding=encoding) as f:
+            f.write(sep.join(data))
+    except:
+        with open(path, 'w') as f:
+            f.write(sep.join(data))
 
 
 def merge_commelement_list(lsts):
@@ -668,15 +817,14 @@ def runtime(func):
     return wrapper
 
 
-def typeassert(*ty_args, **ty_kwargs):
-    """typeassert方法用于强制确认输入格式
+def type_assert(*ty_args, **ty_kwargs):
+    """type_assert方法用于强制确认输入格式
 
-    @typeassert(int, b=str)
+    @type_assert(int, b=str)
     f(a, b)
 
     Parameters
     ----------
-    param : str
 
     Returns
     ----------
@@ -718,6 +866,7 @@ def install(package):
     Returns
     ----------
     """
+    import pip
     if hasattr(pip, 'main'):
         pip.main(['install', package])
     else:
@@ -735,27 +884,33 @@ def flatten(data):
     Returns
     ----------
     """
+    import itertools
     return list(itertools.chain.from_iterable(data))
 
 
-def read_json(path):
+def read_json(path, encoding='UTF-8'):
     """read_json方法用于读取json文件
 
     Parameters
     ----------
     path : str
         json文件路径
-
+    encoding : str
+        编码类型
     Returns
     ----------
     """
     import json
-    with open(path, 'r', encoding='utf-8') as f:
-        data = json.loads(f.read())
+    try:
+        with open(path, 'r', encoding=encoding) as f:
+            data = json.loads(f.read())
+    except:
+        with open(path, 'r') as f:
+            data = json.loads(f.read())
     return data
 
 
-def write_json(data, path, sort=False):
+def write_json(data, path, sort=False, encoding='UTF-8'):
     """write_json方法用于写json到文件中
 
     Parameters
@@ -766,12 +921,18 @@ def write_json(data, path, sort=False):
         保存路径
     sort : bool
         是否排序
+    encoding : str
+        编码类型
     Returns
     ----------
     """
     import json
-    with open(path, "w", encoding='UTF-8') as f:
-        f.write(json.dumps(data, indent=4, ensure_ascii=False, sort_keys=sort))
+    try:
+        with open(path, "w", encoding=encoding) as f:
+            f.write(json.dumps(data, indent=4, ensure_ascii=False, sort_keys=sort))
+    except:
+        with open(path, "w") as f:
+            f.write(json.dumps(data, indent=4, ensure_ascii=False, sort_keys=sort))
 
 
 def show_memory(variable, unit='KB'):
@@ -821,6 +982,26 @@ def lists_compare(l1, l2):
     return dif12_num, dif21_num, inter_num, union_num, dif12, dif21, inter, union_num
 
 
+def iou(l1, l2):
+    """iou方法用于计算交集/并集
+
+    Parameters
+    ----------
+    l1 : list
+        列表1
+    l2 : list
+        列表2
+
+    Returns
+    ----------
+    """
+    inter = inter_set(l1, l2)
+    union = union_set(l1, l2)
+    if len(union) == 0:
+        return 0
+    return len(inter)/len(union)
+
+
 def try_except(func):
     """try_except方法用于try except捕获异常
 
@@ -893,3 +1074,48 @@ def dict_to_list(_dict):
         else:
             ret.append([key, data])
     return ret
+
+
+def write_yaml(data, path, encoding='UTF-8'):
+    """write_yaml方法用于写yaml配置文件
+
+    Parameters
+    ----------
+    data : dict
+        配置字典
+    path : str
+        保存路径
+    encoding : str
+        编码规则
+
+    Returns
+    ----------
+    """
+    import yaml
+    with open(path, 'w', encoding=encoding) as yaml_file:
+        yaml.safe_dump(data, yaml_file)
+
+
+def load_yaml(path, encoding='UTF-8'):
+    """read_yaml方法用于读取yaml文件
+
+    Parameters
+    ----------
+    path : str
+        yaml文件路径
+    encoding : str
+        编码规则
+
+    Returns
+    ----------
+    """
+    import yaml
+    try:
+        with open(path, 'r', encoding=encoding) as f:
+            config = yaml.load(f.read(), Loader=yaml.FullLoader)
+    except:
+        with open(path, 'r') as f:
+            config = yaml.load(f.read())
+    return config
+
+
